/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(__DESKTOPITEM_H)
#define __DESKTOPITEM_H

#include "shlobj.h"

class DesktopItem
{
public:

	DesktopItem(const string &name);
	virtual ~DesktopItem();

	void desktopload(HINSTANCE hInstance, string configLine);
	void desktopreconfigure(string configLine);

	bool isVisible() const { return visible; };

	int getX() const { return actualx; };
	int getY() const { return actualy; };

	const string &getName() const { return name; };

	string getIconCommand() { return iconCommand; };
	string getPositionSaver() { return positionSaver; };
	
	void repaint(bool invalidateCache = false, bool xpMove = false);
	void updateRegion();

	//void setAlwaysOnTop(bool setAlwaysOnTop);
	void reposition(int newx, int newy, int newwidth, int newheight);

	void hide();
	void show();

private:

	string positionSaver;

	HWND hWnd;
	HINSTANCE hInstance;

	HDC backgroundDC;
	HBITMAP backgroundBitmap;

	//bool alwaysOnTop;

	bool iconOverlay;

	bool visible;

	int actualx;
	int actualy;
	int actualwidth;
	int actualheight;

	bool DIsnapto;
	int DIx;
	int DIy;
	int DIwidth;
	int DIheight;
	int DIsnapdistance;

	int align;
	int vertAlign;

	int leftBorder;
	int topBorder;
	int rightBorder;
	int bottomBorder;

	string name;
	
	string text;
	bool noWordBreak;

	char tooltip[MAX_LINE_LENGTH];

	bool mousePressed;
	bool mouseInside;

	bool moveable;
	int moveButton;

	bool easyDesktopFocus;

	string leftClickCommand;
	string leftDoubleClickCommand;
	string middleClickCommand;
	string middleDoubleClickCommand;
	string rightClickCommand;
	string rightDoubleClickCommand;
	string iconCommand;
	string dropCommand;

	POINT savedpt;

	//bool specialDrag;

protected:

	HWND TooltipHints;

	void addHint(HWND hWnd, LPSTR caption);
	void removeHint(HWND hWnd);

	void ontemplateButtonDown(int actualMouseClick);
	void ontemplateButtonUp(const int &x, const int &y, int actualMouseClick, const string &clickCommand);

	int isCloseTo(POINT &pos, const int x, const int y, const int span);

	virtual void onLButtonDblClk(int x, int y);
	virtual void onLButtonDown(int x, int y);
	virtual void onLButtonUp(int x, int y);
	virtual void onMButtonDblClk(int x, int y);
	virtual void onMButtonDown(int x, int y);
	virtual void onMButtonUp(int x, int y);
	virtual void onRButtonDblClk(int x, int y);
	virtual void onRButtonDown(int x, int y);
	virtual void onRButtonUp(int x, int y);
	virtual void onMouseMove(int x, int y);
	virtual void onPaint(HDC hDC);
	virtual void onMove(int x, int y);
	virtual void onSize(int width, int height);
	virtual void onTimer(int timerID);
	virtual void onWindowPosChanging(WINDOWPOS *windowPos);

	virtual bool onWindowMessage(UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult);

public:

	static LRESULT WINAPI windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
};

#endif
