/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "DesktopItem.h"
#include "DesktopTexture.h"

extern DesktopItemList desktopitemList;

DesktopTexture::DesktopTexture()
{
	useSmallIcon = false;

	noActiveIconColor = false;

	NormalGradient.clear();
	ActiveGradient.clear();
}

DesktopTexture::~DesktopTexture()
{
	NormalGradient.clear();
	ActiveGradient.clear();
}

void DesktopTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	int color = 0;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);
		
	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void DesktopTexture::parseGradientCFG(string cfg, int imageType)
{
	if ( cfg.empty() )
		return;

	int color = 0;
	COLORREF insertColor;
	char token[MAX_LINE_LENGTH] = {0};
	LPCSTR nextToken = cfg.c_str();
	
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 0)
		{
			NormalGradient.insert(NormalGradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			ActiveGradient.insert(ActiveGradient.end(), insertColor );
		}
	}
}

void DesktopTexture::configure()
{
	NameValuePair iconTextValues[] = {
		{ "top", 1 },
		{ "bottom", 2 },
		{ "left", 3 },
		{ "right", 4 },
		{ 0, 0 }
	};

	noActiveIconColor = GetxRCBool("xDesktop" , "NoActiveIconColor");

	normal_saturation = GetxRCInt("xDesktop" , "SaturationIntensity", 255, 0, 255);
	normal_hueIntensity = GetxRCInt("xDesktop" , "HueIntensity", 0, 0, 255);
	normal_hueColor = GetxRCColor("xDesktop", "HueColor", RGB(0, 0, 0));

	active_saturation = GetxRCInt("xDesktop" , "ActiveSaturationIntensity", normal_saturation, 0, 255);
	active_hueIntensity = GetxRCInt("xDesktop" , "ActiveHueIntensity", normal_hueIntensity, 0, 255);
	active_hueColor = GetxRCColor("xDesktop", "ActiveHueColor", normal_hueColor);

	iconSize = GetxRCInt("xDesktop", "IconSize", 32, 16, 64);

	iconTextPosition = GetxRCNamedValue("xDesktop", "TextPosition", iconTextValues, 2);
	iconTextSpacing = GetxRCInt("xDesktop" , "TextSpacing", 5);

	useSmallIcon = GetxRCBool("xDesktop", "UseSmallIcon");
	
	string normaltextBG = GetxRCLine("xDesktop", "TextBackground", "ff00ff");
	parseCFG(normaltextBG, iconTextBackground, iconTextBackgroundLight, iconTextBackgroundDark);
	//BG Insert in Gradient List
	NormalGradient.insert(NormalGradient.end(), iconTextBackground );
	
	parseCFG(GetxRCLine("xDesktop", "ActiveTextBackground", normaltextBG), activeiconTextBackground, activeiconTextBackgroundLight, activeiconTextBackgroundDark);
	//BG Insert in Gradient List
	ActiveGradient.insert(ActiveGradient.end(), activeiconTextBackground );

	string normalgradientColor = GetxRCLine("xDesktop", "TextBackgroundGradientColors", "");
	parseGradientCFG(normalgradientColor, 0);	
	normalverticalGradient = GetxRCBool("xDesktop", "TextBackgroundGradientVertical");

	parseGradientCFG(GetxRCLine("xDesktop", "ActiveTextBackgroundGradientColors", normalgradientColor), 1);
	activeverticalGradient = GetxRCBool("xDesktop", "ActiveTextBackgroundGradientVertical", normalverticalGradient);
}

void DesktopTexture::apply(HDC hDC, int x, int y, int width, int height, const string &iconCommand, const string &name, bool activated)
{
	HICON hIcon = NULL;

	SHFILEINFO shFileInfo;
    SHGetFileInfo(iconCommand.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | ( (activated && !noActiveIconColor) ? SHGFI_SELECTED : 0 ) | (useSmallIcon ? SHGFI_SMALLICON : SHGFI_LARGEICON));
	hIcon = shFileInfo.hIcon;
	
	if(hIcon == NULL)
		return;

	HDC	memoryDC = CreateCompatibleDC(hDC);
	HBITMAP memoryBitmap = CreateCompatibleBitmap(hDC, width, height);
	SelectObject(memoryDC, memoryBitmap);

	RECT r;
	r.top = 0;
	r.left = 0;
	r.right = width;
	r.bottom = height;

	::SetBkColor(memoryDC, RGB(255, 0, 255));
	::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

	int xpos, ypos;
	if (iconTextPosition == 1)
	{
		xpos = (width-iconSize)/2;
		ypos = height-iconSize;
		r.top = 0;
		r.left = 0;
		r.right = width;
		r.bottom = height-iconSize-iconTextSpacing;
	}
	else if (iconTextPosition == 2)
	{
		xpos = (width-iconSize)/2;
		ypos = 0;
		r.top = iconSize+iconTextSpacing;
		r.left = 0;
		r.right = width;
		r.bottom = height;
	}
	else if (iconTextPosition == 3)
	{
		xpos = width-iconSize;
		ypos = (height-iconSize)/2;
		r.top = 0;
		r.left = 0;
		r.right = width-iconSize-iconTextSpacing;
		r.bottom = height;
	}
	else if (iconTextPosition == 4)
	{
		xpos = 0;
		ypos = (height-iconSize)/2;
		r.top = 0;
		r.left = iconSize+iconTextSpacing;
		r.right = width;
		r.bottom = height;
	}

	if ( activated )
	{
		::SetBkColor(memoryDC, activeiconTextBackgroundDark);
		::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

		r.right = r.right-1;
		r.bottom = r.bottom-1;
		::SetBkColor(memoryDC, activeiconTextBackgroundLight);
		::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

		r.left = r.left+1;
		r.top = r.top+1;
		if ( ActiveGradient.size() > 1 )
		{
			solidGradientPaint(memoryDC, ActiveGradient, r, activeverticalGradient);
		}
		else
		{
			::SetBkColor(memoryDC, activeiconTextBackground);
			::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
		}
	}
	else
	{
		::SetBkColor(memoryDC, iconTextBackgroundDark);
		::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

		r.right = r.right-1;
		r.bottom = r.bottom-1;
		::SetBkColor(memoryDC, iconTextBackgroundLight);
		::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

		r.left = r.left+1;
		r.top = r.top+1;
		if ( NormalGradient.size() > 1 )
		{
			solidGradientPaint(memoryDC, NormalGradient, r, normalverticalGradient);
		}
		else
		{
			::SetBkColor(memoryDC, iconTextBackground);
			::ExtTextOut(memoryDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
		}
	}

	if ( IsOS(OS_XP) || (normal_saturation != 255 || normal_hueIntensity != 0) || (activated && (active_saturation != 255 || active_hueIntensity != 0)) )
	{
		//Normal
		HDC memoryColorIconDC = CreateCompatibleDC(hDC);
		HBITMAP memoryColorIconBitmap = CreateCompatibleBitmap(hDC, iconSize, iconSize);
		SelectObject(memoryColorIconDC, memoryColorIconBitmap);
		
		DrawIconEx(memoryColorIconDC, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);

		//Mask
		HDC memoryMaskIconDC = CreateCompatibleDC(hDC);
		HBITMAP memoryMaskIconBitmap = CreateCompatibleBitmap(hDC, iconSize, iconSize);
		SelectObject(memoryMaskIconDC, memoryMaskIconBitmap);

		DrawIconEx(memoryMaskIconDC, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_MASK);

		int _saturation = (activated ? active_saturation : normal_saturation);
		int _hueIntensity = (activated ? active_hueIntensity : normal_hueIntensity);
		COLORREF _hueColor = (activated ? active_hueColor : normal_hueColor);

		HDC	memoryPaintDC = CreateCompatibleDC(hDC);
		HBITMAP memoryPaintBitmap = CreateCompatibleBitmap(hDC, iconSize, iconSize);
		SelectObject(memoryPaintDC, memoryPaintBitmap);

		if ( IsOS(OS_XP) )
		{
			int curx = 0;
			int cury = 0;
			for ( DesktopItemListIterator iter = desktopitemList.begin(); iter != desktopitemList.end(); iter++)
			{
				if(stricmp((*iter)->getName().c_str(), name.c_str()) == NULL)
				{
					curx = (*iter)->getX();
					cury = (*iter)->getY();
					break;
				}
			}
			PaintDesktopEx(memoryPaintDC, 0, 0, iconSize, iconSize, curx+xpos, cury+ypos, false);
		}

		int iy = 0;
		int ix = 0;
		COLORREF cl = 0;
		BYTE r = 0;
		BYTE g = 0;
		BYTE b = 0;
		for(iy = 0; iy < iconSize; iy++)
		{
			for(ix = 0; ix < iconSize; ix++)
			{
				// Loop through all transparent pixels...
				// If the mask-pixel is white, then break
				if(!GetPixel(memoryMaskIconDC, ix, iy))
				{
					cl = GetPixel(memoryColorIconDC, ix, iy);

					r = GetRValue(cl);
					g = GetGValue(cl);
					b = GetBValue(cl);

					// Saturation effect
					if(_saturation != 255)
					{
						BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

						r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
						g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
						b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
					}

					// Hue effect
					if(_hueIntensity != 0)
					{
						// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
						// test normal
						r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
						g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
						b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
					}

					SetPixelV(memoryPaintDC, ix, iy, RGB(r, g, b));
				}
				else
					SetPixelV(memoryPaintDC, ix, iy, RGB(255, 0, 255));
			}
		}

		BitBlt( memoryDC, xpos, ypos, iconSize, iconSize, memoryMaskIconDC, 0, 0, SRCAND );
		// Combine the foreground with the background
		BitBlt( memoryDC, xpos, ypos, iconSize, iconSize, memoryPaintDC, 0, 0, (IsOS(OS_XP) ? SRCCOPY : SRCPAINT));

		memoryPaintBitmap = (HBITMAP) SelectObject(memoryPaintDC, memoryPaintBitmap);
		DeleteObject(memoryPaintBitmap);
		DeleteDC(memoryPaintDC);

		memoryColorIconBitmap = (HBITMAP) SelectObject(memoryColorIconDC, memoryColorIconBitmap);
		DeleteObject(memoryColorIconBitmap);
		DeleteDC(memoryColorIconDC);

		memoryMaskIconBitmap = (HBITMAP) SelectObject(memoryMaskIconDC, memoryMaskIconBitmap);
		DeleteObject(memoryMaskIconBitmap);
		DeleteDC(memoryMaskIconDC);
	}
	else
		DrawIconEx(memoryDC, xpos, ypos, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);

	BitBlt( hDC, x, y, width, height, memoryDC, 0, 0, SRCCOPY );

	memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
	DeleteObject(memoryBitmap);
	DeleteDC(memoryDC);

	if ( hIcon )
		DestroyIcon(hIcon);
}

void DesktopTexture::solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, bool vertical)
{
	int nWidth = r.right - r.left;
	int nHeight = r.bottom - r.top;

	RECT r2;
	double steps = 1;
	double cr, cg, cb, dr, dg, db;
	int gradSteps;

	int colorCount = workGradient.size();

	if (vertical)
		gradSteps = max(nHeight / (colorCount-1), 1);
	else
		gradSteps = max(nWidth / (colorCount-1), 1);

	int lastGradientPos;
	if (vertical)
		lastGradientPos = r.top;
	else
		lastGradientPos = r.left;

	for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); ++cit )
	{
		if (cit == workGradient.end())
			break;

		COLORREF one = *cit;
		++cit;
		COLORREF two = *cit;
		--cit;
	
		// get current color
		cr = GetRValue( one );
		cg = GetGValue( one );
		cb = GetBValue( one );

		// Get delta color
		dr = (cr - GetRValue( two )) / gradSteps;
		dg = (cg - GetGValue( two )) / gradSteps;
		db = (cb - GetBValue( two )) / gradSteps;

		for (int i = 1; i <= gradSteps; i++)
		{
			cr -= dr;
			cg -= dg;
			cb -= db;
		
			if (vertical)
				SetRect(&r2, r.left, (int)(lastGradientPos+steps*(i - 1)), r.right, min((int)(lastGradientPos+steps*i), r.bottom));
			else
				SetRect(&r2, (int)(lastGradientPos+steps*(i - 1)), r.top, min((int)(lastGradientPos+steps*i), r.right), r.bottom);

			if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
			{
				::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
				::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
			}
		}
		if (vertical)
			lastGradientPos = r2.bottom;
		else
			lastGradientPos = r2.right;
	}
}
